@@ -391,7 +391,8 @@ class Data(object):
 
         if isinstance(label, ComponentID):
             component_id = label
-            component_id.parent = self
+            if component_id.parent is None:
+                component_id.parent = self
         else:
             component_id = ComponentID(label, hidden=hidden, parent=self)
 
@@ -471,7 +472,7 @@ class Data(object):
         :rtype: list
         """
         return [cid for cid, comp in self._components.items()
-                if not cid.hidden and not comp.hidden]
+                if not cid.hidden and not comp.hidden and cid.parent is self]
 
     @property
     def coordinate_components(self):
@@ -528,9 +529,18 @@ class Data(object):
             label in the primary and one in the derived components, the primary
             one takes precedence.
         """
+
         for cid_set in (self.primary_components, self.derived_components):
-            result = [cid for cid in cid_set if
-                      cid.label == label or cid is label]
+
+            result = []
+            for cid in cid_set:
+                if isinstance(label, ComponentID):
+                    if cid is label:
+                        result.append(cid)
+                else:
+                    if cid.label == label:
+                        result.append(cid)
+
             if len(result) == 1:
                 return result[0]
             elif len(result) > 1:
@@ -734,7 +744,10 @@ class Data(object):
             else:
                 s += "Main "
             s += "components:\n"
-            components = [c for c in self.components if c.hidden == hidden]
+            if hidden:
+                components = [c for c in self.components if c not in self.visible_components]
+            else:
+                components = [c for c in self.visible_components]
             for i, cid in enumerate(components):
                 if cid.hidden != hidden:
                     continue
@@ -922,7 +935,7 @@ class Data(object):
             if cname in new_labels - old_labels:
                 cid = data.find_component_id(cname)
                 comp_new = data.get_component(cname)
-                self.add_component(comp_new, cid)
+                self.add_component(comp_new, cid.label)
 
         # Update data label
         self.label = data.label
